;*****************************************************************************
; Script Name:  DlrToTxt
;     Purpose:  Converts Paradox currency (or number) field
;               to a text string for printing on a check
;      Author:  John C. Frykland
;               White Oak Software
;               PO Box 760
;               San Carlos, CA 94070
;               (415) 637-0222
;*****************************************************************************
;   This code is copyright (c) 1992 by White Oak Software
;   It has been adapted from an Accounts Payable application
;   originally written in RPG II on an IBM System/34
;
;   You may use this code in your applications without royalty
;   or consideration under the following conditions:
;
;   1.  You DO NOT copyright or claim this code as your own
;   2.  You DO NOT charge others for this code alone
;   3.  You DO include the copyright notice for all to see
;   4.  You DO hold White Oak Software and John C. Frykland
;       completely harmless from any direct, incidental or
;       other damages resulting from the use of this code.
;       This includes, but is not limited to, damages resulting
;       from incorrectly printed checks.
;
;   Method of Use:
;   1.  Call wosInitDlrToTxt to initialize the arrays used by
;       the conversion function
;   2.  Call wosCnvDlrToTxt passing the dollar amount to convert;
;       the function will return a text string
;       e.g. TextStr = wosCnvDlrToTxt([Check Amount])
;
;   Warning:
;   1.  The function can handle amounts up to 999,999,999,999.99 only
;   2.  The function does not handle negative numbers;
;       the sign will be ignored
;*****************************************************************************

;-----------------------------------------------------------------------------
; procedure   : wosInitDlrToTxt
; description : Define global variables used by DlrToTxt routines
; arguments   : none
; returns     : none
;-----------------------------------------------------------------------------
PROC wosInitDlrToTxt()
    Array OnesArr[19]
    OnesArr[1]  = "One "
    OnesArr[2]  = "Two "
    OnesArr[3]  = "Three "
    OnesArr[4]  = "Four "
    OnesArr[5]  = "Five "
    OnesArr[6]  = "Six "
    OnesArr[7]  = "Seven "
    OnesArr[8]  = "Eight "
    OnesArr[9]  = "Nine "
    OnesArr[10] = "Ten "
    OnesArr[11] = "Eleven "
    OnesArr[12] = "Twelve "
    OnesArr[13] = "Thirteen "
    OnesArr[14] = "Fourteen "
    OnesArr[15] = "Fifteen "
    OnesArr[16] = "Sixteen "
    OnesArr[17] = "Seventeen "
    OnesArr[18] = "Eighteen "
    OnesArr[19] = "Nineteen "

    Array TensArr[9]
    TensArr[1] = "Ten "
    TensArr[2] = "Twenty "
    TensArr[3] = "Thirty "
    TensArr[4] = "Forty "
    TensArr[5] = "Fifty "
    TensArr[6] = "Sixty "
    TensArr[7] = "Seventy "
    TensArr[8] = "Eighty "
    TensArr[9] = "Ninety "

    Array HunsArr[4]
    HunsArr[1] = "Billion "
    HunsArr[2] = "Million "
    HunsArr[3] = "Thousand "
    HunsArr[4] = "Hundred "
ENDPROC
WriteLib LibName wosInitDlrToTxt
Release Procs wosInitDlrToTxt

;-----------------------------------------------------------------------------
; procedure   : wosCnvDlrToTxt
; description : Converts currency value from a numeric value
;             : to a text string for printing on a check
; arguments   : DlrVal - dollar value to be converted
; returns     : DlrStr - text string created by this routine
;-----------------------------------------------------------------------------
PROC wosCnvDlrToTxt(DlrVal)
    Private DlrStr,          ;; high order text string to return
            CentStr,         ;; low order text string to return
            StrPtr,          ;; work pointer for SubStr proc
            WorkVal,         ;; temporary work area
            LoopX            ;; controls loop thru value string

    DlrVal  = Format("w16.2", DlrVal)
    CentStr = "and " + SubStr(DlrVal, 15, 2) + "/100"
        ;; works by taking groups of 3 digits from number string
        ;; and determining what the textual value will be
        ;; string to be converted is in following format:
        ;; sBBBMMMTTTHHH.CC
        ;;    where s = sign
        ;;          B = billions
        ;;          M = millions
        ;;          T = thousands
        ;;          H = hundreds
        ;;          C = cents
    StrPtr = 2               ;; skip over sign
    DlrStr = ""
    For LoopX from 1 to ArraySize(HunsArr)
        WorkVal = NumVal(SubStr(DlrVal, StrPtr, 3))
        StrPtr  = StrPtr + 3
            ;; current 3 digits are blank or zero
        If WorkVal = "Error" or WorkVal = 0 then
            Loop
        EndIf
            ;; we have value in hundreds
        If WorkVal > 99 then
            DlrStr  = DlrStr + OnesArr[Int(WorkVal / 100)] + "Hundred "
            WorkVal = Mod(WorkVal, 100)
        EndIf
            ;; we have value in tens (at least 20)
        If WorkVal > 19 then
            DlrStr = DlrStr + TensArr[Int(WorkVal / 10)]
            WorkVal = Mod(WorkVal, 10)
        EndIf
            ;; we have value between 1 and 19
        If WorkVal > 0 then
            DlrStr = DlrStr + OnesArr[WorkVal]
        EndIf
            ;; add billion, million or thousand
        If LoopX < ArraySize(HunsArr) then
            DlrStr = DlrStr + HunsArr[LoopX]
        EndIf
    EndFor
        ;; in case we have no significant digits
    If DlrStr = "" then
        DlrStr = "Zero "
    EndIf
    Return (DlrStr + CentStr)
ENDPROC
WriteLib LibName wosCnvDlrToTxt
Release Procs wosCnvDlrToTxt

